@@ -30,7 +30,7 @@ sub new
     $self->{symboltable}        = undef;
     $self->{verbosity}          = 0;
     $self->{level}              = -1;
-    $self->{outputfilename}     = "-";
+    $self->{outputfilename}     = '-';
     $self->{outf}               = undef;
     $self->{rules}              = {};
     $self->{subrules}           = {};
@@ -39,17 +39,17 @@ sub new
     return $self;
 }
 
-sub setVerbosity         { my ($self, $verbosity) = @_; $self->{verbosity} = defined($verbosity) ? $verbosity : 0;   }
-sub symboltable          { my ($self) = @_; return $self->{symboltable};                                             }
-sub setoutputfile        { my ($self, $outputfilename) = @_; $self->{outputfilename} = $outputfilename;              }
+sub setVerbosity         { my ($self, $verbosity) = @_; $self->{verbosity} = defined($verbosity) ? $verbosity : 0; }
+sub symboltable          { my ($self) = @_; return $self->{symboltable};                                           }
+sub setoutputfile        { my ($self, $outputfilename) = @_; $self->{outputfilename} = $outputfilename;            }
 
-sub fragment2class       { my ($self) = @_; $self->{generationoptions}{fragment2class} = 'true';                     }
-sub embedactions         { my ($self) = @_; $self->{generationoptions}{embedactions} = 'true';                       }
-sub stripactions         { my ($self) = @_; $self->{generationoptions}{stripactions} = 'true';                       }
-sub stripallcomments     { my ($self) = @_; $self->{generationoptions}{stripallcomments} = 'true';                   }
-sub matchcaseinsensitive { my ($self) = @_; $self->{generationoptions}{matchcaseinsensitive} = 'true';               }
-sub shiftlazytogreedy    { my ($self) = @_; $self->{generationoptions}{shiftlazytogreedy} = 'true';                  }
-sub buildkeywords        { my ($self) = @_; $self->{generationoptions}{buildkeywords} = 'true';                      }
+sub fragment2class       { my ($self) = @_; $self->{generationoptions}{fragment2class}       = 'true';             }
+sub embedactions         { my ($self) = @_; $self->{generationoptions}{embedactions}         = 'true';             }
+sub stripactions         { my ($self) = @_; $self->{generationoptions}{stripactions}         = 'true';             }
+sub stripallcomments     { my ($self) = @_; $self->{generationoptions}{stripallcomments}     = 'true';             }
+sub matchcaseinsensitive { my ($self) = @_; $self->{generationoptions}{matchcaseinsensitive} = 'true';             }
+sub shiftlazytogreedy    { my ($self) = @_; $self->{generationoptions}{shiftlazytogreedy}    = 'true';             }
+sub buildkeywords        { my ($self) = @_; $self->{generationoptions}{buildkeywords}        = 'true';             }
 
 sub testoption
 {
@@ -95,7 +95,7 @@ sub addMarpaRule
 }
 
 ##
-#   deleteMarpaRule : delete 'subrulename' if it is a subrule of 'rulename'
+#   deleteMarpaRule : delete '$rulename'
 ##
 sub deleteMarpaRule
 {
@@ -105,8 +105,26 @@ sub deleteMarpaRule
 
     mydie "INTERNAL: can't delete undefined rule $rulename" if !exists $rules->{$rulename};
 
-    $rules->{$rulename} = { deletedafterconversiontoclass => "true" };
     $self->deleteAllSubrules($rulename);
+    $rules->{$rulename} = { deletedafterconversiontoclass => 'true' };
+}
+
+##
+#   deleteAllSubrules : delete all subrules of '$rulename'
+##
+sub deleteAllSubrules
+{
+    my ($self, $rulename ) = @_;
+
+    my $subrules    = $self->{subrules};
+    my $rules       = $self->{rules};
+
+    return if !exists $subrules->{$rulename};
+
+    for my $subrulename (@{$subrules->{$rulename}})
+    {
+        $rules->{$subrulename} = { deletedafterconversiontoclass => 'true' };
+    }
 }
 
 ##
@@ -120,6 +138,9 @@ sub deleteSingleSubrule
 
     my $subrules = $self->{subrules}{$rulename};
 
+    ##
+    #   remove the subrule by creating a new list of subrules
+    ##
     my $status = 0;
     my @newrules;
 
@@ -139,7 +160,7 @@ sub deleteSingleSubrule
     {
         my $rules = $self->{rules};
         mydie "INTERNAL: can't delete undefined rule $subrulename" if !exists $rules->{$subrulename};
-        $rules->{$subrulename} = { deletedafterconversiontoclass => "true" }
+        $rules->{$subrulename} = { deletedafterconversiontoclass => 'true' }
     }
 }
 
@@ -175,7 +196,7 @@ sub tagMarpaRule
 
     my $rules = $self->{rules};
     mydie "INTERNAL: rule $rulename is undefined" if !exists $rules->{$rulename};
-    $rules->{$rulename}{status} = "done";
+    $rules->{$rulename}{status} = 'done';
 }
 
 sub isSubrule
@@ -217,30 +238,11 @@ sub exitLevel
     $context->{level} = $level;
 }
 
-sub generateSubruleName
-{
-    my ($self, $rulename, $cardinality, $context) = @_;
-
-    $context->{subruleindex}{$rulename}{index} = 0 if !exists $context->{subruleindex}{$rulename}{index};
-
-    my $subruleindex = ++$context->{subruleindex}{$rulename}{index};
-    my $subrulename  = sprintf "%s_%03d", $rulename, $subruleindex;
-
-    if ( defined $cardinality && $cardinality eq "?" )
-    {
-        $subrulename = "opt_" . $subrulename;
-    }
-
-    push @{$self->{subrules}{$rulename}}, $subrulename;
-
-    return $subrulename;
-}
-
 sub  retrieveModifier
 {
     my ($token, $tag) = @_;
 
-    return undef if ref $token ne "HASH";
+    return undef if ref $token ne 'HASH';
 
     my $modifier = undef;
     $modifier = $token->{$tag} if exists $token->{$tag};
@@ -257,7 +259,7 @@ sub negateElement
     #     printf "found!\n";
     # }
 
-    mydie "INTERNAL: negated element is not a hash in rule $rulename" if ref $mxtoken ne "HASH";
+    mydie "INTERNAL: negated element is not a hash in rule $rulename" if ref $mxtoken ne 'HASH';
 
     my $mxelement;
 
@@ -443,6 +445,25 @@ sub mergeKeywordFragment
     }
 }
 
+sub generateSubruleName
+{
+    my ($self, $rulename, $cardinality, $context) = @_;
+
+    $context->{subruleindex}{$rulename}{index} = 0 if !exists $context->{subruleindex}{$rulename}{index};
+
+    my $subruleindex = ++$context->{subruleindex}{$rulename}{index};
+    my $subrulename  = sprintf "%s_%03d", $rulename, $subruleindex;
+
+    if ( defined $cardinality && $cardinality eq "?" )
+    {
+        $subrulename = "opt_" . $subrulename;
+    }
+
+    push @{$self->{subrules}{$rulename}}, $subrulename;
+
+    return $subrulename;
+}
+
 ##
 #   createSubRule :     create a (non-)anonymous subrule to 'rulename'.
 #                       create a negated class if the class or token group was negated.
@@ -487,21 +508,6 @@ sub createSubRule
     return $mxalias;
 }
 
-sub deleteAllSubrules
-{
-    my ($self, $rulename ) = @_;
-
-    my $subrules    = $self->{subrules};
-    my $rules       = $self->{rules};
-
-    return if !exists $subrules->{$rulename};
-
-    for my $subrulename (@{$subrules->{$rulename}})
-    {
-        $rules->{$subrulename} = { deletedafterconversiontoclass => "true" };
-    }
-}
-
 ## ------------------------------------------------------
 # 'walktoken' : process all token flavours
 ## ------------------------------------------------------
@@ -520,7 +526,7 @@ sub walktoken
     my $mxToken     = {};
     my $comments    = [];
 
-    if (ref $token eq "HASH" && exists $token->{comment} && !$self->testoption('stripallcomments'))
+    if (ref $token eq 'HASH' && exists $token->{comment} && !$self->testoption('stripallcomments'))
     {
         mydie "comment not a plain string in rule $rulename" if ref $token->{comment} ne "";
         my $cl = $self->renderComment($rulename, [$token->{comment}]);
@@ -538,14 +544,14 @@ sub walktoken
             $negation = undef;
             last SWITCH;
         };
-        (ref $token eq "HASH" && exists $token->{type} && $token->{type} eq "tokengroup") && do {
+        (ref $token eq 'HASH' && exists $token->{type} && $token->{type} eq 'tokengroup') && do {
             $mxToken = $self->walkgroup($rulename, $token, $context);
             $mxToken = $self->createSubRule( $rulename, $mxToken, $negation, $cardinality, $context );
             # reset 'negation' since it was processed by 'createSubRule'
             $negation = undef;
             last SWITCH;
         };
-        (ref $token eq "HASH" && exists $token->{type} && $token->{type} eq "rulegroup") && do {
+        (ref $token eq 'HASH' && exists $token->{type} && $token->{type} eq 'rulegroup') && do {
             $mxToken = $self->walkgroup($rulename, $token, $context);
             # strip lexeme status off rule groups
             delete $mxToken->{isLexeme};
@@ -554,49 +560,38 @@ sub walktoken
             $negation = undef;
             last SWITCH;
         };
-        (ref $token eq "HASH" && exists $token->{type} && $token->{type} eq "literal") && do {
+        (ref $token eq 'HASH' && exists $token->{type} && $token->{type} eq 'literal') && do {
             my $tokenvalue = $token->{value};
-            # if ( !$self->testoption('matchcaseinsensitive') || !$self->isAlphaLiteral($tokenvalue) )
-            # {
-                $mxToken = { type => 'literal', value => $tokenvalue, isLexeme => 1 };
-                my $class4list = convertLiteralToClass($tokenvalue);
-                $mxToken->{class4list} = $class4list if defined $class4list;
-            # }
-            # else
-            # {
-            #     my $literaltoken = generatematchcaseinsensitive($tokenvalue);
-            #     delete $self->{generationoptions}{matchcaseinsensitive};
-            #     $mxToken = $self->walkgroup($rulename, $literaltoken, $context);
-            #     $mxToken = $self->createSubRule( $rulename, $mxToken, $negation, $cardinality, $context );
-            #     $self->{generationoptions}{matchcaseinsensitive} = 'true';
-            # }
+            $mxToken = { type => 'literal', value => $tokenvalue, isLexeme => 1 };
+            my $class4list = convertLiteralToClass($tokenvalue);
+            $mxToken->{class4list} = $class4list if defined $class4list;
             last SWITCH;
         };
-        (ref $token eq "HASH" && exists $token->{type} && $token->{type} eq "class") && do {
+        (ref $token eq 'HASH' && exists $token->{type} && $token->{type} eq 'class') && do {
             $mxToken = { type => 'class', value => $token->{value}, isLexeme => 1, class4list => $token->{value} };
             last SWITCH;
         };
-        (ref $token eq "HASH" && exists $token->{type} && $token->{type} eq "regex") && do {
+        (ref $token eq 'HASH' && exists $token->{type} && $token->{type} eq 'regex') && do {
             $mxToken = { type => 'unsupported', msg => "can't convert arbitrary regex to Marpa in $rulename" };
             last SWITCH;
         };
-        (ref $token eq "HASH" && exists $token->{type} && $token->{type} eq "range") && do {
+        (ref $token eq 'HASH' && exists $token->{type} && $token->{type} eq 'range') && do {
             $mxToken = convertRangeToClass($token);
             last SWITCH;
         };
-        (ref $token eq "HASH" && exists $token->{type} && $token->{type} eq "value") && do {
+        (ref $token eq 'HASH' && exists $token->{type} && $token->{type} eq 'value') && do {
             $mxToken = { type => 'unsupported', msg => "can't convert 'value' to Marpa in $rulename" };
             last SWITCH;
         };
-        (ref $token eq "HASH" && exists $token->{token}) && do {
+        (ref $token eq 'HASH' && exists $token->{token}) && do {
             $mxToken = $self->walktoken($rulename, $token->{token}, $context);
             last SWITCH;
         };
-        (ref $token eq "HASH" && exists $token->{action}) && do {
+        (ref $token eq 'HASH' && exists $token->{action}) && do {
             $mxToken = { type => 'ignore', msg => "embedded action in $rulename" };
             last SWITCH;
         };
-        (ref $token eq "HASH" && exists $token->{comment}) && do {
+        (ref $token eq 'HASH' && exists $token->{comment}) && do {
             $mxToken = { type => 'ignore', msg => "comment in $rulename" };
             last SWITCH;
         };
@@ -639,7 +634,7 @@ sub walknonterminal
 {
     my ( $self, $rulename, $nonterminal, $context ) = @_;
 
-    mydie "nonterminal is not a hash for rule $rulename" if ref $nonterminal ne "HASH";
+    mydie "nonterminal is not a hash for rule $rulename" if ref $nonterminal ne 'HASH';
 
     my $mxToken = {};
 
@@ -661,7 +656,7 @@ sub walknonterminal
 
 ## -------------------------------------------------------------------------------
 # 'retagSimpleRule' : check if the cardinality can be added to 'mxtoken'.
-#  Rules  :
+#  Conditions :
 #  - '?' is consumed by 'opt_' subrules, so we have to reject it here
 #  - 'mxtoken' must be a subrule of 'rule'
 #  - the parent rule must consist of a single element
@@ -671,19 +666,21 @@ sub retagSimpleRule
 {
     my ( $self, $rulename, $mxtoken, $cardinality, $context, $options ) = @_;
 
-    mydie "token must be a hash in $rulename"   if ref $mxtoken ne "HASH";
+    mydie "retagSimpleRule : token must be a hash in $rulename" if ref $mxtoken ne 'HASH';
     return 0 if $cardinality eq "?";
 
     $cardinality =~ s/([+*])\?/$1/ if $cardinality =~ /[+*]\?/ && $self->testoption('shiftlazytogreedy');
-    mydie sprintf "lazy quantifier %s in rule %s not supported by Marpa", $cardinality, $rulename if $cardinality =~ /[+*]\?/;
+    mydie sprintf "lazy quantifier %s in rule %s not supported by Marpa. use option -g to map to greedy.", $cardinality, $rulename if $cardinality =~ /[+*]\?/;
 
-    my $abortatfirstlevel = defined $options && ref $options eq "HASH" && exists $options->{abortatfirstlevel};
+    $options = {} if !defined $options;
+
+    my $abortatfirstlevel = exists $options->{abortatfirstlevel};
 
     SWITCH: {
         (exists $mxtoken->{rhs}) && do {
             my $rhs = $mxtoken->{rhs};
 
-            mydie "'rhs' must be a hash in $rulename"   if ref $rhs ne "HASH";
+            mydie "'rhs' must be a hash in $rulename"   if ref $rhs ne 'HASH';
             mydie "'token' must be a scalar in 'rhs' in rule $rulename" if !exists $rhs->{token} || ref $rhs->{token} ne "";
 
             return 0 if !defined $self->checkMarpaRuleExists($rhs->{token});
@@ -731,7 +728,7 @@ sub retagSimpleRule
             my $ec = 0;
             for my $al (@{$mxtoken->{list}})
             {
-                mydie "alternative list is not an array in rule $rulename" if ref $al ne "HASH" || !exists $al->{list};
+                mydie "alternative list is not an array in rule $rulename" if ref $al ne 'HASH' || !exists $al->{list};
                 for my $le (@{$al->{list}})
                 {
                     mydie "alternative element is not simple in rule $rulename" if exists $le->{type} && $le->{type} !~ /literal|class/;
@@ -741,7 +738,7 @@ sub retagSimpleRule
             return $ec <= 1;
             last SWITCH;
         };
-        (exists $mxtoken->{type} && $mxtoken->{type} eq "literal") && do {
+        (exists $mxtoken->{type} && $mxtoken->{type} eq 'literal') && do {
             $mxtoken->{cardinality} = $cardinality;
             return 1;
             last SWITCH;
@@ -861,8 +858,8 @@ sub processRightSides
 
     for my $e (@$rhslist)
     {
-        $e = $e->{rhs}          if ref $e eq "HASH" && exists $e->{rhs}  && !exists $e->{token};
-        $e = { token => $e }    if ref $e eq "HASH" && exists $e->{type} &&  exists $e->{definition};
+        $e = $e->{rhs}          if ref $e eq 'HASH' && exists $e->{rhs}  && !exists $e->{token};
+        $e = { token => $e }    if ref $e eq 'HASH' && exists $e->{type} &&  exists $e->{definition};
 
         my $negation    = retrieveModifier($e, 'negation');
         my $cardinality = retrieveModifier($e, 'cardinality');
@@ -887,7 +884,7 @@ sub processRightSides
             next;
         }
 
-        if (ref $e eq "HASH" && !exists $e->{token})
+        if (ref $e eq 'HASH' && !exists $e->{token})
         {
             $self->abortWithError( "generic regex rules not supported in rule $rulename", $e ) if exists $e->{type} && $e->{type} eq "regex";
             $self->abortWithError( "INTERNAL: 'e' must be a hash and contain 'token' in rule $rulename", $e );
@@ -911,7 +908,7 @@ sub processRightSides
                 if ($self->testoption("buildkeywords") && $keywordfragment->{status} == 0 && $keyword =~ /^[a-z]/i)
                 {
                     $currentlist = [
-                        { type => "literal", value => $keyword, isLexeme => 1 }
+                        { type => 'literal', value => $keyword, isLexeme => 1 }
                     ];
                     $newlist = { list => $currentlist, isLexeme => 1 };
                 }
@@ -957,7 +954,7 @@ sub processRightSides
         }
         else
         {
-            $self->abortWithError( "can't process group for rule $rulename", $rhslist ) if ref $e ne "HASH" || !exists $e->{token};
+            $self->abortWithError( "can't process group for rule $rulename", $rhslist ) if ref $e ne 'HASH' || !exists $e->{token};
             $mxelement = $self->walktoken($rulename, $e->{token}, $context);
             # ignore embedded comments/actions
             next if exists $mxelement->{type} && $mxelement->{type} eq "ignore";
@@ -1007,11 +1004,12 @@ sub processRightSides
             if ( scalar @$rhslist <= 1 && $self->retagSimpleRule($rulename, $mxelement, $cardinality, $context) )
             {
                 ##
-                #   if the tagged element is part of an alternative branch, we have to make it into a subrule
+                #   if the tagged element is part of an alternative branch (i.e. a branch that has more than one element),
+                #   we have to make it into a subrule
                 ##
-                if (defined $alternative)
+                if (defined $alternative && scalar @$rhslist > 1)
                 {
-                    $mxelement = $self->createSubRule( $rulename, $mxelement, undef, undef, $context ) if scalar @$rhslist > 1;
+                    $mxelement = $self->createSubRule( $rulename, $mxelement, undef, undef, $context );
                 }
             }
             else
@@ -1061,7 +1059,7 @@ sub processRightSides
     if ($self->testoption("buildkeywords") && $keywordfragment->{status} == 0 && $keywordfragment->{value} =~ /^[a-z]/i)
     {
         $currentlist = [
-            { type => "literal", value => $keywordfragment->{value}, isLexeme => 1 }
+            { type => 'literal', value => $keywordfragment->{value}, isLexeme => 1 }
         ];
         $groupisLexeme->{status} = 0;
     }
@@ -1100,7 +1098,7 @@ sub walkgroup
 {
     my ($self, $rulename, $tokengroup, $context) = @_;
 
-    $self->abortWithError( "INTERNAL: group is missing 'definition' in rule $rulename", $tokengroup ) if ref $tokengroup ne "HASH" || !exists $tokengroup->{definition};
+    $self->abortWithError( "INTERNAL: group is missing 'definition' in rule $rulename", $tokengroup ) if ref $tokengroup ne 'HASH' || !exists $tokengroup->{definition};
     my $definition  = $tokengroup->{definition};
     my $grouplist   = $self->processRightSides( $rulename, $definition, $context );
 
@@ -1125,24 +1123,26 @@ sub walkrule
     # test if we traversed this node already
     if ( defined $rulestatus )
     {
-        if ($rulestatus eq "synthetic" && !$self->checkMarpaRuleExists($rulename))
-        {
-            my $mxrightside = $self->processRightSides( $rulename, $rule->{rightsides}, $context );
-            $self->addMarpaRule($rulename, $mxrightside);
-            $symboltable->tagrule($rulename, 'done');
-            $rulestatus  = $symboltable->rulestatus($rulename);
-        }
-
         my $mxrule;
 
         SWITCH: {
             ($rulestatus eq "inprogress") && do {$mxrule = $symboltable->rule( $rulename ); last SWITCH; };
             ($rulestatus eq "done")       && do {$mxrule = $self->getMarpaRule( undef, $rulename ); last SWITCH; };
+            ($rulestatus eq "synthetic" && !$self->checkMarpaRuleExists($rulename)) && do
+            {
+                my $mxrightside = $self->processRightSides( $rulename, $rule->{rightsides}, $context );
+                $self->addMarpaRule($rulename, $mxrightside);
+                $symboltable->tagrule($rulename, 'done');
+                $rulestatus     = $symboltable->rulestatus($rulename);
+                $mxrule         = {};
+                last SWITCH;
+            };
             do { mydie "unexpected rule status $rulestatus in rule $rulename" };
         }
 
         my $result              = { mxrule => $rulename, rhs => { token => $rulename } };
         $result->{class4list}   = $mxrule->{class4list} if exists $mxrule->{class4list};
+
         return $result;
     }
 
@@ -1152,10 +1152,10 @@ sub walkrule
     my $rightside   = $rule->{rightsides};
     if (defined $rightside)
     {
-        $self->abortWithError( "rhs is not an array ref in $rulename", $rightside ) if ref $rightside ne "ARRAY";
+        $self->abortWithError( "rhs is not an array ref in $rulename", $rightside ) if ref $rightside ne 'ARRAY';
 
-        $context->{subruleindex}{$rulename}{grammarstate}       = $rule->{grammarstate} if exists $rule->{grammarstate} && $rule->{grammarstate} eq "lexer";
-        $context->{subruleindex}{$rulename}{isFragmentOrChild}  = 'true'                if exists $rule->{type}         && $rule->{type}         eq "fragment";
+        $context->{subruleindex}{$rulename}{grammarstate}      = $rule->{grammarstate} if exists $rule->{grammarstate} && $rule->{grammarstate} eq "lexer";
+        $context->{subruleindex}{$rulename}{isFragmentOrChild} = 'true'                if exists $rule->{type}         && $rule->{type}         eq "fragment";
 
         ##
         #  prevent infinite recursion by tagging any symbol
@@ -1178,6 +1178,7 @@ sub walkrule
     }
 
     $self->addMarpaRule($rulename, $mxrightside);
+
     $self->exitLevel($rulename, $context);
 
     return $mxrightside;
@@ -1350,7 +1351,7 @@ sub renderComment
 {
     my ($self, $rulename, $comments) = @_;
 
-    mydie "INTERNAL : 'comments' is not an array in rule $rulename" if ref $comments ne "ARRAY";
+    mydie "INTERNAL : 'comments' is not an array in rule $rulename" if ref $comments ne 'ARRAY';
 
     my $result      = [];
     for my $comment (@$comments)
@@ -1415,7 +1416,43 @@ sub writeFragmentAsClass
     printf $outf "%s\n", $outputline;
 }
 
+#
+#   translateunicode :  translate unicodes embedded in literals or classes.
+#                       die if we are in codepages beyond 00
 ##
+sub translateunicode
+{
+    my ($self, $rulename, $string ) = @_;
+
+    while ( $string =~ /(\\u([0-9A-F]{4,4}))|(\\u\{([0-9A-F]+)\})/i )
+    {
+        my $fourmatch   = $2;
+        my $bracedmatch = $4;
+
+        SWITCH: {
+            (defined $fourmatch) && do {
+                mydie "translateunicode : class string $string not in codepage 00 in $rulename" if $fourmatch !~ /00([0-9A-Z][0-9A-Z])/i;
+                my $translatedcode = $1;
+                $string =~ s/\\u${fourmatch}/\\x${translatedcode}/i;
+                last SWITCH;
+            };
+            (defined $bracedmatch) && do {
+                mydie "translateunicode : class string $string not in codepage 00 in $rulename" if $bracedmatch !~ /0*([0-9A-Z]{2,2})/i;
+                my $translatedcode = $1;
+                $string =~ s/\\u\{${bracedmatch}\}/\\x${translatedcode}/i;
+                last SWITCH;
+            };
+            do {
+                mydie "translateunicode : unexpected match";
+                last SWITCH;
+            };
+        }
+    }
+
+    return $string;
+}
+
+#
 #   normalizeClassString :  remove class elements that are redundant when the class is made case-insensitive
 #                           return the modified class text as well as an indicator if the class contains letters
 ##
@@ -1434,14 +1471,14 @@ sub normalizeClassString
     while ( $classstring ne "" )
     {
         SWITCH: {
-        ($classstring =~ /(\\u[0-9a-f]+)-(\\u[0-9a-f]+)(.*)$/i) && do {
+        ($classstring =~ /(\\[ux][0-9a-f]+)-(\\[ux][0-9a-f]+)(.*)$/i) && do {
             my $beg = $1;
             my $end = $2;
             $result .= "${1}-${2}";
             $classstring = $3;
             last SWITCH;
         };
-        ($classstring =~ /(\\u[0-9a-f]{4,4})(.*)$/i) && do {
+        ($classstring =~ /(\\[ux][0-9a-f]{2,4})(.*)$/i) && do {
             my $c = $1;
             $result .= $c;
             $classstring = $2;
@@ -1522,29 +1559,31 @@ sub computeRhs
 {
     my ($self, $rulename, $rhs, $options ) = @_;
 
-    my $isLexeme = 1 if exists $rhs->{isLexeme};
-
     # if ($rulename eq "HEXNUMBER_003")
     # {
     #     printf "found!\n";
     # }
 
-    SWITCH: {
-        (exists $rhs->{type} && $rhs->{type} eq "negatedclass") && do {
+    SWITCH:
+    {
+        (exists $rhs->{type} && $rhs->{type} eq 'negatedclass') && do {
             mydie "computeRhs : 'value' missing in negatedclass $rulename" if !exists $rhs->{value};
-            $rhs = { value => "[" . $rhs->{value} . "]" };
+            my $value = $rhs->{value};
+            $value = $self->translateunicode($rulename, $value);
+            $rhs = { value => "[${value}]" };
             last SWITCH;
         };
         (exists $rhs->{deletedafterconversiontoclass}) && do {
             $rhs = { discard => 'true' };
             last SWITCH;
         };
-        (exists $rhs->{type} && $rhs->{type} eq "unsupported") && do {
+        (exists $rhs->{type} && $rhs->{type} eq 'unsupported') && do {
             mydie $rhs->{msg};
             last SWITCH;
         };
-        (exists $rhs->{type} && $rhs->{type} eq "class") && do {
+        (exists $rhs->{type} && $rhs->{type} eq 'class') && do {
             my $value = $rhs->{value};
+            $value = $self->translateunicode($rulename, $value);
             my $modifier = "";
             if ($self->testoption('matchcaseinsensitive') && $self->isAlphaLiteral($value))
             {
@@ -1555,8 +1594,9 @@ sub computeRhs
             $rhs = { value => "[${value}]${modifier}" };
             last SWITCH;
         };
-        (exists $rhs->{type} && $rhs->{type} eq "literal") && do {
+        (exists $rhs->{type} && $rhs->{type} eq 'literal') && do {
             my $literal = $rhs->{value};
+            $literal = $self->translateunicode($rulename, $literal);
             $literal =  ($literal eq "\\'") ? "[']" : "'${literal}'";
             if ($self->testoption('matchcaseinsensitive') && $self->isAlphaLiteral($literal))
             {
@@ -1566,7 +1606,7 @@ sub computeRhs
             $rhs = { value => $literal };
             last SWITCH;
         };
-        (exists $rhs->{type} && $rhs->{type} eq "action") && do {
+        (exists $rhs->{type} && $rhs->{type} eq 'action') && do {
             push @{$options->{comments}}, " # Action: " . $rhs->{token}{action};
             $rhs = { discard => 'true' };
             last SWITCH;
@@ -1613,7 +1653,7 @@ sub printRhs
 
     return "" if exists $rhs->{discard};
 
-    mydie "'options' not well-defined" if ref $options ne "HASH" || !exists $options->{delimiter};
+    mydie "'options' not well-defined" if ref $options ne 'HASH' || !exists $options->{delimiter};
 
     my $delimiter       = $options->{delimiter};
 
@@ -1639,7 +1679,8 @@ sub printRhs
         $cardinality = "";
     }
 
-    SWITCH: {
+    SWITCH:
+    {
         ($status == 0) && do {
             $result .= sprintf "%s %-3s %s%s", $self->pad($rulename), $definitionop, $rhs->{value}, $cardinality;
             last SWITCH;
@@ -1667,7 +1708,7 @@ sub writeMarpaRuleList
 {
     my ( $self, $rulename, $rule, $options ) = @_;
 
-    # if ($rulename eq "MINUSCHAR")
+    # if ($rulename eq "SAFECODEPOINT")
     # {
     #     printf "found!\n";
     # }
@@ -1691,7 +1732,7 @@ sub writeMarpaRuleList
 
             for my $cl (@{$metalist->{comments}})
             {
-                $cl = $cl->[0] if ref $cl eq "ARRAY";
+                $cl = $cl->[0] if ref $cl eq 'ARRAY';
                 printf $outf "%s\n", $cl;
             }
             if (exists $metalist->{actions})
@@ -1700,7 +1741,7 @@ sub writeMarpaRuleList
                 {
                     for my $cl (@{$metalist->{actions}})
                     {
-                        $cl = $cl->[0] if ref $cl eq "ARRAY";
+                        $cl = $cl->[0] if ref $cl eq 'ARRAY';
                         printf $outf "%s\n", $cl;
                     }
                 }
@@ -1709,7 +1750,7 @@ sub writeMarpaRuleList
                     $actiontext = "";
                     for my $cl (@{$metalist->{actions}})
                     {
-                        $cl = $cl->[0] if ref $cl eq "ARRAY";
+                        $cl = $cl->[0] if ref $cl eq 'ARRAY';
                         $actiontext .= $cl;
                     }
                 }
@@ -1734,7 +1775,8 @@ sub writeMarpaRuleList
 
             my $nextclause;
 
-            SWITCH: {
+            SWITCH:
+            {
                 (exists $rhs->{type} && $rhs->{type} eq "group" && exists $rhs->{list}) && do {
                     if ( exists $rhs->{class4list})
                     {
@@ -1923,7 +1965,7 @@ sub convertRedirectToDiscard
 {
     my ($self) = @_;
 
-    return if !exists $self->{discarded} || ref $self->{discarded} ne "ARRAY" || scalar @{$self->{discarded}} < 1;
+    return if !exists $self->{discarded} || ref $self->{discarded} ne 'ARRAY' || scalar @{$self->{discarded}} < 1;
 
     my $outf = $self->{outf};
 
@@ -1966,9 +2008,8 @@ sub writeMarpaGrammar
     $self->generateGenericOptions;
     $self->generateStartClause;
 
-    # create a ':discard' rule from the rules
-    # tagged 'redirect' at the top of the
-    # output file.
+    # create a ':discard' rule at the top of the output file
+    # from the rules tagged 'redirect'
     $self->convertRedirectToDiscard;
 
     my $outf = $self->{outf};
@@ -1985,7 +2026,7 @@ sub writeMarpaGrammar
         if (exists $rule->{comment} && !$self->testoption('stripallcomments'))
         {
             my $comment = $rule->{comment};
-            $comment = [$comment] if ref $comment ne "ARRAY";
+            $comment = [$comment] if ref $comment ne 'ARRAY';
             my $cl = $self->renderComment("grammar", $comment);
             for my $s (@$cl)
             {
